@@ -33,14 +33,16 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <glib/gi18n-lib.h>
-#include <gmodule.h>
 #include <string.h>
-
+#include <libpeas/peas-extension-base.h>
+#include <libpeas/peas-object-module.h>
+#include <libpeas/peas-activatable.h>
 #include <unistd.h>
 #include <lirc/lirc_client.h>
 
 #include "totem-plugin.h"
 #include "totem.h"
+#include "totem-dirs.h"
 
 #define TOTEM_TYPE_LIRC_PLUGIN		(totem_lirc_plugin_get_type ())
 #define TOTEM_LIRC_PLUGIN(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), TOTEM_TYPE_LIRC_PLUGIN, TotemLircPlugin))
@@ -51,7 +53,7 @@
 
 typedef struct
 {
-	TotemPlugin   parent;
+	PeasExtensionBase parent;
 
 	GIOChannel *lirc_channel;
 	struct lirc_config *lirc_config;
@@ -61,7 +63,7 @@ typedef struct
 
 typedef struct
 {
-	TotemPluginClass parent_class;
+	PeasExtensionBaseClass parent_class;
 } TotemLircPluginClass;
 
 /* strings that we recognize as commands from lirc */
@@ -85,8 +87,6 @@ typedef struct
 #define TOTEM_IR_COMMAND_PLAYPAUSE "play_pause"
 #define TOTEM_IR_COMMAND_ZOOM_UP "zoom_up"
 #define TOTEM_IR_COMMAND_ZOOM_DOWN "zoom_down"
-#define TOTEM_IR_COMMAND_SHOW_PLAYING "show_playing"
-#define TOTEM_IR_COMMAND_SHOW_VOLUME "show_volume"
 #define TOTEM_IR_COMMAND_EJECT "eject"
 #define TOTEM_IR_COMMAND_PLAY_DVD "play_dvd"
 #define TOTEM_IR_COMMAND_MUTE "mute"
@@ -96,26 +96,15 @@ typedef struct
 #define TOTEM_IR_SETTING_TOGGLE_REPEAT "setting_repeat"
 #define TOTEM_IR_SETTING_TOGGLE_SHUFFLE "setting_shuffle"
 
-G_MODULE_EXPORT GType register_totem_plugin	(GTypeModule *module);
 GType	totem_lirc_plugin_get_type		(void) G_GNUC_CONST;
 
 static void totem_lirc_plugin_init		(TotemLircPlugin *plugin);
-static void totem_lirc_plugin_finalize		(GObject *object);
-static gboolean impl_activate			(TotemPlugin *plugin, TotemObject *totem, GError **error);
-static void impl_deactivate			(TotemPlugin *plugin, TotemObject *totem);
 
-TOTEM_PLUGIN_REGISTER(TotemLircPlugin, totem_lirc_plugin)
+TOTEM_PLUGIN_REGISTER(TOTEM_TYPE_LIRC_PLUGIN, TotemLircPlugin, totem_lirc_plugin)
 
 static void
 totem_lirc_plugin_class_init (TotemLircPluginClass *klass)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	TotemPluginClass *plugin_class = TOTEM_PLUGIN_CLASS (klass);
-
-	object_class->finalize = totem_lirc_plugin_finalize;
-
-	plugin_class->activate = impl_activate;
-	plugin_class->deactivate = impl_deactivate;
 }
 
 static void
@@ -123,12 +112,6 @@ totem_lirc_plugin_init (TotemLircPlugin *plugin)
 {
 }
 
-static void
-totem_lirc_plugin_finalize (GObject *object)
-{
-	G_OBJECT_CLASS (totem_lirc_plugin_parent_class)->finalize (object);
-}
-
 static char *
 totem_lirc_get_url (const char *str)
 {
@@ -198,10 +181,6 @@ totem_lirc_to_command (const gchar *str, char **url)
 		return TOTEM_REMOTE_COMMAND_ZOOM_UP;
 	else if (strcmp (str, TOTEM_IR_COMMAND_ZOOM_DOWN) == 0)
 		return TOTEM_REMOTE_COMMAND_ZOOM_DOWN;
-	else if (strcmp (str, TOTEM_IR_COMMAND_SHOW_PLAYING) == 0)
-		return TOTEM_REMOTE_COMMAND_SHOW_PLAYING;
-	else if (strcmp (str, TOTEM_IR_COMMAND_SHOW_VOLUME) == 0)
-		return TOTEM_REMOTE_COMMAND_SHOW_VOLUME;
 	else if (strcmp (str, TOTEM_IR_COMMAND_EJECT) == 0)
 		return TOTEM_REMOTE_COMMAND_EJECT;
 	else if (strcmp (str, TOTEM_IR_COMMAND_PLAY_DVD) == 0)
@@ -270,12 +249,12 @@ totem_lirc_read_code (GIOChannel *source, GIOCondition condition, TotemLircPlugi
 	return TRUE;
 }
 
-static gboolean
-impl_activate (TotemPlugin *plugin,
-	       TotemObject *totem,
-	       GError **error)
+static void
+impl_activate (PeasActivatable *plugin,
+	       GObject *object)
 {
 	TotemLircPlugin *pi = TOTEM_LIRC_PLUGIN (plugin);
+	TotemObject *totem = TOTEM_OBJECT (object);
 	char *path;
 	int fd;
 
@@ -283,19 +262,25 @@ impl_activate (TotemPlugin *plugin,
 
 	fd = lirc_init ("Totem", 0);
 	if (fd < 0) {
-		g_set_error (error, TOTEM_PLUGIN_ERROR, TOTEM_PLUGIN_ERROR_ACTIVATION,
-				_("Couldn't initialize lirc."));
+		//FIXME
+#if 0
+		g_set_error_literal (error, TOTEM_PLUGIN_ERROR, TOTEM_PLUGIN_ERROR_ACTIVATION,
+                                     _("Couldn't initialize lirc."));
 		return FALSE;
+#endif
 	}
 
 	/* Load the default Totem setup */
-	path = totem_plugin_find_file (plugin, "totem_lirc_default");
+	path = totem_plugin_find_file ("lirc", "totem_lirc_default");
 	if (path == NULL || lirc_readconfig (path, &pi->lirc_config, NULL) == -1) {
 		g_free (path);
-		g_set_error (error, TOTEM_PLUGIN_ERROR, TOTEM_PLUGIN_ERROR_ACTIVATION,
-			     _("Couldn't read lirc configuration."));
+		//FIXME
+#if 0
+		g_set_error_literal (error, TOTEM_PLUGIN_ERROR, TOTEM_PLUGIN_ERROR_ACTIVATION,
+                                     _("Couldn't read lirc configuration."));
+#endif
 		close (fd);
-		return FALSE;
+		return;
 	}
 	g_free (path);
 
@@ -307,13 +292,11 @@ impl_activate (TotemPlugin *plugin,
 	g_io_channel_set_buffered (pi->lirc_channel, FALSE);
 	g_io_add_watch (pi->lirc_channel, G_IO_IN | G_IO_ERR | G_IO_HUP,
 			(GIOFunc) totem_lirc_read_code, pi);
-
-	return TRUE;
 }
 
 static void
-impl_deactivate	(TotemPlugin *plugin,
-		 TotemObject *totem)
+impl_deactivate (PeasActivatable *plugin,
+		 GObject *object)
 {
 	TotemLircPlugin *pi = TOTEM_LIRC_PLUGIN (plugin);
 	GError *error = NULL;
@@ -340,3 +323,4 @@ impl_deactivate	(TotemPlugin *plugin,
 		pi->totem = NULL;
 	}
 }
+
