@@ -28,11 +28,14 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <glib/gi18n-lib.h>
+#include <libpeas/peas-extension-base.h>
+#include <libpeas/peas-object-module.h>
+#include <libpeas/peas-activatable.h>
 #include <gdata/gdata.h>
-#include <libsoup/soup.h>
 
 #include "totem-plugin.h"
 #include "totem.h"
+#include "totem-dirs.h"
 #include "totem-video-list.h"
 #include "totem-interface.h"
 #include "backend/bacon-video-widget.h"
@@ -59,10 +62,9 @@ enum {
 #define TOTEM_YOUTUBE_PLUGIN_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), TOTEM_TYPE_YOUTUBE_PLUGIN, TotemYouTubePluginClass))
 
 typedef struct {
-	TotemPlugin parent;
+	PeasExtensionBase parent;
 	Totem *totem;
 	GDataYouTubeService *service;
-	SoupSession *session;
 	BaconVideoWidget *bvw;
 
 	guint current_tree_view;
@@ -85,17 +87,14 @@ typedef struct {
 } TotemYouTubePlugin;
 
 typedef struct {
-	TotemPluginClass parent_class;
+	PeasExtensionBaseClass parent_class;
 } TotemYouTubePluginClass;
 
-G_MODULE_EXPORT GType register_totem_plugin	(GTypeModule *module);
 GType totem_youtube_plugin_get_type		(void) G_GNUC_CONST;
-
-static gboolean impl_activate			(TotemPlugin *plugin, TotemObject *totem, GError **error);
-static void impl_deactivate			(TotemPlugin *plugin, TotemObject *totem);
+static void totem_youtube_plugin_class_init	(TotemYouTubePluginClass *klass);
 
 /* GtkBuilder callbacks */
-void notebook_switch_page_cb (GtkNotebook *notebook, gpointer *page, guint page_num, TotemYouTubePlugin *self);
+void notebook_switch_page_cb (GtkNotebook *notebook, GtkNotebookPage *page, guint page_num, TotemYouTubePlugin *self);
 void search_button_clicked_cb (GtkButton *button, TotemYouTubePlugin *self);
 void cancel_button_clicked_cb (GtkButton *button, TotemYouTubePlugin *self);
 void search_entry_activate_cb (GtkEntry *entry, TotemYouTubePlugin *self);
@@ -105,27 +104,22 @@ void open_in_web_browser_activate_cb (GtkAction *action, TotemYouTubePlugin *sel
 void value_changed_cb (GtkAdjustment *adjustment, TotemYouTubePlugin *self);
 gboolean starting_video_cb (TotemVideoList *video_list, GtkTreePath *path, TotemYouTubePlugin *self);
 
-TOTEM_PLUGIN_REGISTER (TotemYouTubePlugin, totem_youtube_plugin)
+TOTEM_PLUGIN_REGISTER(TOTEM_TYPE_YOUTUBE_PLUGIN, TotemYouTubePlugin, totem_youtube_plugin);
 
 static void
 totem_youtube_plugin_class_init (TotemYouTubePluginClass *klass)
 {
-	TotemPluginClass *plugin_class = TOTEM_PLUGIN_CLASS (klass);
-
-	plugin_class->activate = impl_activate;
-	plugin_class->deactivate = impl_deactivate;
 }
 
 static void
 totem_youtube_plugin_init (TotemYouTubePlugin *plugin)
 {
-	/* Nothing to see here; move along */
 }
 
 /* ----------------------------------------------------------------------------------------------------------------- */
 /* Copied from http://bugzilla.gnome.org/show_bug.cgi?id=575900 while waiting for them to be committed to gdk-pixbuf */
 
-typedef	struct {
+typedef struct {
 	gint width;
 	gint height;
 	gboolean preserve_aspect_ratio;
@@ -334,10 +328,11 @@ set_up_tree_view (TotemYouTubePlugin *self, GtkBuilder *builder, guint key)
 	self->cancel_button = GTK_WIDGET (gtk_builder_get_object (builder, "yt_cancel_button"));
 }
 
-static gboolean
-impl_activate (TotemPlugin *plugin, TotemObject *totem, GError **error)
+static void
+impl_activate (PeasActivatable *plugin, GObject *object)
 {
 	TotemYouTubePlugin *self = TOTEM_YOUTUBE_PLUGIN (plugin);
+	TotemObject *totem = TOTEM_OBJECT (object);
 	GtkWindow *main_window;
 	GtkBuilder *builder;
 	guint i;
@@ -347,7 +342,7 @@ impl_activate (TotemPlugin *plugin, TotemObject *totem, GError **error)
 
 	/* Set up the interface */
 	main_window = totem_get_main_window (totem);
-	builder = totem_plugin_load_interface (plugin, "youtube.ui", TRUE, main_window, self);
+	builder = totem_plugin_load_interface ("youtube", "youtube.ui", TRUE, main_window, self);
 	g_object_unref (main_window);
 
 	self->search_entry = GTK_ENTRY (gtk_builder_get_object (builder, "yt_search_entry"));
@@ -365,12 +360,10 @@ impl_activate (TotemPlugin *plugin, TotemObject *totem, GError **error)
 	/* Add the sidebar page */
 	totem_add_sidebar_page (totem, "youtube", _("YouTube"), self->vbox);
 	g_object_unref (builder);
-
-	return TRUE;
 }
 
 static void
-impl_deactivate	(TotemPlugin *plugin, TotemObject *totem)
+impl_deactivate (PeasActivatable *plugin, GObject *totem)
 {
 	guint i;
 	TotemYouTubePlugin *self = TOTEM_YOUTUBE_PLUGIN (plugin);
@@ -378,10 +371,10 @@ impl_deactivate	(TotemPlugin *plugin, TotemObject *totem)
 	totem_remove_sidebar_page (self->totem, "youtube");
 
 	for (i = 0; i < NUM_TREE_VIEWS; i++) {
-		/* Cancel any queries which are still underway */
-		if (self->cancellable[i] != NULL)
+		if (self->cancellable[i] != NULL) {
 			g_cancellable_cancel (self->cancellable[i]);
-
+			g_object_unref (self->cancellable[i]);
+		}
 		if (self->query[i] != NULL)
 			g_object_unref (self->query[i]);
 	}
@@ -390,14 +383,20 @@ impl_deactivate	(TotemPlugin *plugin, TotemObject *totem)
 		g_object_unref (self->playing_video);
 	if (self->service != NULL)
 		g_object_unref (self->service);
-	if (self->session != NULL)
-		g_object_unref (self->session);
 	g_object_unref (self->bvw);
 	g_object_unref (self->totem);
 	if (self->regex != NULL)
 		g_regex_unref (self->regex);
 }
 
+static const gchar *
+get_fmt_param (TotemYouTubePlugin *self)
+{
+	if (bacon_video_widget_get_connection_speed (self->bvw) >= 10)
+		return "&fmt=18";
+	return "";
+}
+
 typedef struct {
 	TotemYouTubePlugin *plugin;
 	guint tree_view;
@@ -457,137 +456,79 @@ increment_progress_bar_fraction (TotemYouTubePlugin *self, guint tree_view)
 		gdk_window_set_cursor (gtk_widget_get_window (self->vbox), NULL);
 		gtk_progress_bar_set_text (self->progress_bar[tree_view], "");
 		gtk_progress_bar_set_fraction (self->progress_bar[tree_view], 0.0);
+
+		/* Disable the "Cancel" button, if it applies to the current tree view */
+		if (self->current_tree_view == tree_view)
+			gtk_widget_set_sensitive (self->cancel_button, FALSE);
+
+		/* Unref cancellable */
+		if (self->cancellable[tree_view] != NULL)
+			g_object_unref (self->cancellable[tree_view]);
+		self->cancellable[tree_view] = NULL;
 	}
 }
 
 typedef struct {
 	TotemYouTubePlugin *plugin;
 	GDataEntry *entry;
-	GtkTreePath *path;
+	GtkTreeIter iter;
 	guint tree_view;
-	SoupMessage *message;
-	gulong cancelled_id;
-	GCancellable *cancellable;
 } TParamData;
 
-/* Ranked list of formats to prefer, from http://en.wikipedia.org/wiki/YouTube#Quality_and_codecs */
-static const guint fmt_preferences[] = {
-	17, /* least preferred (lowest connection speed needed) */
-	5,
-	18,
-	18,
-	34,
-	35,
-	43,
-	43,
-	22,
-	45,
-	45,
-	37,
-	38 /* most preferred (highest connection speed needed) */
-};
-
 static void
-resolve_t_param_cb (SoupSession *session, SoupMessage *message, TParamData *data)
+resolve_t_param_cb (GObject *source_object, GAsyncResult *result, TParamData *data)
 {
-	gchar *video_uri = NULL;
-	const gchar *video_id, *contents;
+	gchar *contents, *video_uri = NULL;
+	const gchar *video_id;
 	gsize length;
 	GMatchInfo *match_info;
-	GtkTreeIter iter;
+	GError *error = NULL;
 	TotemYouTubePlugin *self = data->plugin;
 
-	/* Prevent cancellation */
-	g_cancellable_disconnect (data->cancellable, data->cancelled_id);
-
 	/* Finish loading the page */
-	if (message->status_code != SOUP_STATUS_OK) {
+	if (g_file_load_contents_finish (G_FILE (source_object), result, &contents, &length, NULL, &error) == FALSE) {
 		GtkWindow *window;
 
 		/* Bail out if the operation was cancelled */
-		if (message->status_code == SOUP_STATUS_CANCELLED)
+		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) == TRUE) {
+			g_error_free (error);
 			goto free_data;
+		}
 
 		/* Couldn't load the page contents; error */
 		window = totem_get_main_window (data->plugin->totem);
-		totem_interface_error (_("Error Looking Up Video URI"), message->response_body->data, window);
+		totem_interface_error (_("Error Looking Up Video URI"), error->message, window);
 		g_object_unref (window);
+		g_error_free (error);
 		goto free_data;
 	}
 
-	contents = message->response_body->data;
-	length = message->response_body->length;
-
 	video_id = gdata_youtube_video_get_video_id (GDATA_YOUTUBE_VIDEO (data->entry));
 
-	/* Check for the fmt_url_map parameter */
+	/* Check for the t parameter, which is now in a JavaScript array on the video page */
 	g_regex_match (self->regex, contents, 0, &match_info);
 	if (g_match_info_matches (match_info) == TRUE) {
-		gchar *fmt_url_map_escaped, *fmt_url_map;
-		gchar **mappings, **i;
-		GHashTable *fmt_table;
-		gint connection_speed;
+		gchar *t_param, *s;
+		const gchar *fmt_param;
+		GString *video_uri_string;
 
 		/* We have a match */
-		fmt_url_map_escaped = g_match_info_fetch (match_info, 1);
-		fmt_url_map = g_uri_unescape_string (fmt_url_map_escaped, NULL);
-		g_free (fmt_url_map_escaped);
+		s = g_match_info_fetch (match_info, 1);
+		t_param = g_uri_unescape_string (s, NULL);
+		if (t_param == NULL)
+			t_param = s;
+		else
+			g_free (s);
+		fmt_param = get_fmt_param (self);
 
-		/* The fmt_url_map parameter is in the following format:
-		 *   fmt1|uri1,fmt2|uri2,fmt3|uri3,...
-		 * where fmtN is an identifier for the audio and video encoding and resolution as described here:
-		 * (http://en.wikipedia.org/wiki/YouTube#Quality_and_codecs) and uriN is the playback URI for that format.
-		 *
-		 * We parse it into a hash table from format to URI, and use that against a ranked list of preferred formats (based on the user's
-		 * connection speed) to determine the URI to use. */
-		fmt_table = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, g_free);
-		mappings = g_strsplit (fmt_url_map, ",", 0);
+		video_uri_string = g_string_new ("http://www.youtube.com/get_video?video_id=");
+		g_string_append_uri_escaped (video_uri_string, video_id, NULL, TRUE);
+		g_string_append (video_uri_string, "&t=");
+		g_string_append_uri_escaped (video_uri_string, t_param, NULL, TRUE);
+		g_string_append (video_uri_string, fmt_param);
 
-		for (i = mappings; *i != NULL; i++) {
-			/* For the moment we just take the first format we get */
-			gchar **mapping;
-			gint fmt;
-
-			mapping = g_strsplit (*i, "|", 2);
-
-			if (mapping[0] == NULL || mapping[1] == NULL) {
-				g_warning ("Bad format-URI mapping: %s", *i);
-				g_strfreev (mapping);
-				continue;
-			}
-
-			fmt = atoi (mapping[0]);
-
-			if (fmt < 1) {
-				g_warning ("Badly-formed format: %s", mapping[0]);
-				g_strfreev (mapping);
-				continue;
-			}
-
-			g_hash_table_insert (fmt_table, GUINT_TO_POINTER ((guint) fmt), g_strdup (mapping[1]));
-			g_strfreev (mapping);
-		}
-
-		g_strfreev (mappings);
-
-		/* Starting with the highest connection speed we support, look for video URIs matching our connection speed. */
-		connection_speed = MIN (bacon_video_widget_get_connection_speed (self->bvw), (gint) G_N_ELEMENTS (fmt_preferences) - 1);
-		for (; connection_speed >= 0; connection_speed--) {
-			guint idx = (guint) connection_speed;
-			video_uri = g_strdup (g_hash_table_lookup (fmt_table, GUINT_TO_POINTER (fmt_preferences [idx])));
-
-			/* Have we found a match yet? */
-			if (video_uri != NULL) {
-				g_debug ("Using video URI for format %u (connection speed %u)", fmt_preferences[idx], idx);
-				break;
-			}
-		}
-
-		g_hash_table_destroy (fmt_table);
-	}
-
-	/* Fallback */
-	if (video_uri == NULL) {
+		video_uri = g_string_free (video_uri_string, FALSE);
+	} else {
 		GDataMediaContent *content;
 
 		/* We don't have a match, which is odd; fall back to the FLV URI as advertised by the YouTube API */
@@ -602,14 +543,12 @@ resolve_t_param_cb (SoupSession *session, SoupMessage *message, TParamData *data
 			video_uri = NULL;
 		}
 	}
-
 	g_match_info_free (match_info);
+	g_free (contents);
 
 	/* Update the tree view with the new MRL */
-	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (self->list_store[data->tree_view]), &iter, data->path) == TRUE) {
-		gtk_list_store_set (self->list_store[data->tree_view], &iter, 2, video_uri, -1);
-		g_debug ("Updated list store with new video URI (\"%s\") for entry %s", video_uri, video_id);
-	}
+	gtk_list_store_set (self->list_store[data->tree_view], &(data->iter), 2, video_uri, -1);
+	g_debug ("Updated list store with new video URI (\"%s\") for entry %s", video_uri, video_id);
 
 	g_free (video_uri);
 
@@ -617,49 +556,37 @@ free_data:
 	/* Update the progress bar */
 	increment_progress_bar_fraction (self, data->tree_view);
 
-	g_object_unref (data->cancellable);
 	g_object_unref (data->plugin);
 	g_object_unref (data->entry);
-	gtk_tree_path_free (data->path);
 	g_slice_free (TParamData, data);
 }
 
 static void
-resolve_t_param_cancelled_cb (GCancellable *cancellable, TParamData *data)
+resolve_t_param (TotemYouTubePlugin *self, GDataEntry *entry, GtkTreeIter *iter, guint tree_view)
 {
-	/* This will cause resolve_t_param_cb() to be called, which will free the data */
-	soup_session_cancel_message (data->plugin->session, data->message, SOUP_STATUS_CANCELLED);
-}
-
-static void
-resolve_t_param (TotemYouTubePlugin *self, GDataEntry *entry, GtkTreeIter *iter, guint tree_view, GCancellable *cancellable)
-{
-	GDataLink *page_link;
+	GDataLink *link;
+	GFile *video_page;
 	TParamData *data;
 
 	/* We have to get the t parameter from the actual HTML video page, since Google changed how their URIs work */
-	page_link = gdata_entry_look_up_link (entry, GDATA_LINK_ALTERNATE);
-	g_assert (page_link != NULL);
+	link = gdata_entry_look_up_link (entry, GDATA_LINK_ALTERNATE);
+	g_assert (link != NULL);
 
 	data = g_slice_new (TParamData);
 	data->plugin = g_object_ref (self);
 	data->entry = g_object_ref (entry);
-	data->path = gtk_tree_model_get_path (GTK_TREE_MODEL (self->list_store[tree_view]), iter);
+	data->iter = *iter;
 	data->tree_view = tree_view;
-	data->cancellable = g_object_ref (cancellable);
 
-	data->message = soup_message_new (SOUP_METHOD_GET, gdata_link_get_uri (page_link));
-	data->cancelled_id = g_cancellable_connect (cancellable, (GCallback) resolve_t_param_cancelled_cb, data, NULL);
-
-	/* Send the message. Consumes a reference to data->message after resolve_t_param_cb() finishes */
-	soup_session_queue_message (self->session, data->message, (SoupSessionCallback) resolve_t_param_cb, data);
+	video_page = g_file_new_for_uri (gdata_link_get_uri (link));
+	g_file_load_contents_async (video_page, self->cancellable[tree_view], (GAsyncReadyCallback) resolve_t_param_cb, data);
+	g_object_unref (video_page);
 }
 
 typedef struct {
 	TotemYouTubePlugin *plugin;
-	GtkTreePath *path;
+	GtkTreeIter iter;
 	guint tree_view;
-	GCancellable *cancellable;
 } ThumbnailData;
 
 static void
@@ -667,7 +594,6 @@ thumbnail_loaded_cb (GObject *source_object, GAsyncResult *result, ThumbnailData
 {
 	GdkPixbuf *thumbnail;
 	GError *error = NULL;
-	GtkTreeIter iter;
 	TotemYouTubePlugin *self = data->plugin;
 
 	/* Finish loading the thumbnail */
@@ -689,10 +615,8 @@ thumbnail_loaded_cb (GObject *source_object, GAsyncResult *result, ThumbnailData
 	g_debug ("Finished creating thumbnail from stream");
 
 	/* Update the tree view */
-	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (self->list_store[data->tree_view]), &iter, data->path) == TRUE) {
-		gtk_list_store_set (self->list_store[data->tree_view], &iter, 0, thumbnail, -1);
-		g_debug ("Updated list store with new thumbnail");
-	}
+	gtk_list_store_set (self->list_store[data->tree_view], &(data->iter), 0, thumbnail, -1);
+	g_debug ("Updated list store with new thumbnail");
 
 	g_object_unref (thumbnail);
 
@@ -701,8 +625,6 @@ free_data:
 	increment_progress_bar_fraction (self, data->tree_view);
 
 	g_object_unref (data->plugin);
-	g_object_unref (data->cancellable);
-	gtk_tree_path_free (data->path);
 	g_slice_free (ThumbnailData, data);
 }
 
@@ -712,6 +634,7 @@ thumbnail_opened_cb (GObject *source_object, GAsyncResult *result, ThumbnailData
 	GFile *thumbnail_file;
 	GFileInputStream *input_stream;
 	GError *error = NULL;
+	TotemYouTubePlugin *self = data->plugin;
 
 	/* Finish opening the thumbnail */
 	thumbnail_file = G_FILE (source_object);
@@ -724,40 +647,20 @@ thumbnail_opened_cb (GObject *source_object, GAsyncResult *result, ThumbnailData
 		return;
 	}
 
-	/* NOTE: There's no need to reset the cancellable before using it again, as we'll have bailed before now if it was ever cancelled. */
-
 	g_debug ("Creating thumbnail from stream");
 	totem_gdk_pixbuf_new_from_stream_at_scale_async (G_INPUT_STREAM (input_stream), THUMBNAIL_WIDTH, -1, TRUE,
-							 data->cancellable, (GAsyncReadyCallback) thumbnail_loaded_cb, data);
+							 self->cancellable[data->tree_view], (GAsyncReadyCallback) thumbnail_loaded_cb, data);
 	g_object_unref (input_stream);
 }
 
 typedef struct {
 	TotemYouTubePlugin *plugin;
 	guint tree_view;
-	GCancellable *query_cancellable;
-	GCancellable *t_param_cancellable;
-	GCancellable *thumbnail_cancellable;
 } QueryData;
 
-static void
-query_data_free (QueryData *data)
-{
-	if (data->thumbnail_cancellable != NULL)
-		g_object_unref (data->thumbnail_cancellable);
-	if (data->t_param_cancellable != NULL)
-		g_object_unref (data->t_param_cancellable);
-
-	g_object_unref (data->query_cancellable);
-	g_object_unref (data->plugin);
-
-	g_slice_free (QueryData, data);
-}
-
 static void
 query_finished_cb (GObject *source_object, GAsyncResult *result, QueryData *data)
 {
-	GtkWindow *window;
 	GDataFeed *feed;
 	GError *error = NULL;
 	TotemYouTubePlugin *self = data->plugin;
@@ -770,45 +673,38 @@ query_finished_cb (GObject *source_object, GAsyncResult *result, QueryData *data
 	self->progress_bar_increment[data->tree_view] = 1.0;
 	increment_progress_bar_fraction (self, data->tree_view);
 
-	if (feed != NULL) {
-		/* Success! */
-		g_object_unref (feed);
-		query_data_free (data);
+	if (feed == NULL) {
+		GtkWindow *window;
 
-		return;
+		/* Bail out if the operation was cancelled */
+		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) == TRUE) {
+			g_error_free (error);
+			goto free_data;
+		}
+
+		/* Error! */
+		window = totem_get_main_window (data->plugin->totem);
+		if (g_error_matches (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR) == TRUE) {
+			/* Hide the ugly technical message libgdata gives behind a nice one telling them it's out of date (which it likely is
+			 * if we're receiving a protocol error). */
+			totem_interface_error (_("Error Searching for Videos"),
+					       _("The response from the server could not be understood. "
+					         "Please check you are running the latest version of libgdata."), window);
+		} else {
+			/* Spew out the error message as provided */
+			totem_interface_error (_("Error Searching for Videos"), error->message, window);
+		}
+
+		g_object_unref (window);
+		g_error_free (error);
+		goto free_data;
 	}
 
-	/* Bail out if the operation was cancelled */
-	if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) == TRUE) {
-		/* Cancel the t-param and thumbnail threads, if applicable */
-		if (data->t_param_cancellable != NULL)
-			g_cancellable_cancel (data->t_param_cancellable);
-		if (data->thumbnail_cancellable != NULL)
-			g_cancellable_cancel (data->thumbnail_cancellable);
+	g_object_unref (feed);
 
-		goto finish;
-	}
-
-	/* Error! */
-	window = totem_get_main_window (data->plugin->totem);
-	if (g_error_matches (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR) == TRUE) {
-		/* Hide the ugly technical message libgdata gives behind a nice one telling them it's out of date (which it likely is
-		 * if we're receiving a protocol error). */
-		totem_interface_error (_("Error Searching for Videos"),
-				       _("The response from the server could not be understood. "
-				         "Please check you are running the latest version of libgdata."), window);
-	} else {
-		/* Spew out the error message as provided */
-		totem_interface_error (_("Error Searching for Videos"), error->message, window);
-	}
-
-	g_object_unref (window);
-
-finish:
-	g_error_free (error);
-	query_data_free (data);
-
-	return;
+free_data:
+	g_object_unref (data->plugin);
+	g_slice_free (QueryData, data);
 }
 
 static void
@@ -822,6 +718,9 @@ query_progress_cb (GDataEntry *entry, guint entry_key, guint entry_count, QueryD
 	GtkProgressBar *progress_bar;
 	TotemYouTubePlugin *self = data->plugin;
 
+	/* Check this query hasn't finished */
+	g_assert (self->cancellable[data->tree_view] != NULL);
+
 	/* Add the entry to the tree view */
 	title = gdata_entry_get_title (entry);
 	id = gdata_youtube_video_get_video_id (GDATA_YOUTUBE_VIDEO (entry));
@@ -843,9 +742,7 @@ query_progress_cb (GDataEntry *entry, guint entry_key, guint entry_count, QueryD
 	gtk_progress_bar_set_fraction (progress_bar, gtk_progress_bar_get_fraction (progress_bar) + self->progress_bar_increment[data->tree_view]);
 
 	/* Resolve the t parameter for the video, which is required before it can be played */
-	/* This will be cancelled if the main query is cancelled, in query_finished_cb() */
-	data->t_param_cancellable = g_cancellable_new ();
-	resolve_t_param (self, entry, &iter, data->tree_view, data->t_param_cancellable);
+	resolve_t_param (self, entry, &iter, data->tree_view);
 
 	/* Download the entry's thumbnail, ready for adding it to the tree view.
 	 * Find the thumbnail size which is closest to the wanted size (THUMBNAIL_WIDTH), so that we:
@@ -877,84 +774,52 @@ query_progress_cb (GDataEntry *entry, guint entry_key, guint entry_count, QueryD
 
 		t_data = g_slice_new (ThumbnailData);
 		t_data->plugin = g_object_ref (self);
-		t_data->path = gtk_tree_model_get_path (GTK_TREE_MODEL (self->list_store[data->tree_view]), &iter);
+		t_data->iter = iter;
 		t_data->tree_view = data->tree_view;
 
-		/* We can use the same cancellable for reading the file and making a pixbuf out of it, as they're consecutive operations */
-		/* This will be cancelled if the main query is cancelled, in query_finished_cb() */
-		data->thumbnail_cancellable = g_cancellable_new ();
-		t_data->cancellable = g_object_ref (data->thumbnail_cancellable);
-
 		g_debug ("Starting thumbnail download for entry %s", id);
 		thumbnail_file = g_file_new_for_uri (gdata_media_thumbnail_get_uri (thumbnail));
-		g_file_read_async (thumbnail_file, G_PRIORITY_DEFAULT, data->thumbnail_cancellable,
+		g_file_read_async (thumbnail_file, G_PRIORITY_DEFAULT, self->cancellable[data->tree_view],
 				   (GAsyncReadyCallback) thumbnail_opened_cb, t_data);
 		g_object_unref (thumbnail_file);
 	}
 }
 
-/* Called when self->cancellable[tree_view] is destroyed (for either tree view) */
-static void
-cancellable_notify_cb (TotemYouTubePlugin *self, GCancellable *old_cancellable)
-{
-	guint i;
-
-	/* Disable the "Cancel" button, if it applies to the current tree view */
-	if (self->cancellable[self->current_tree_view] == old_cancellable)
-		gtk_widget_set_sensitive (self->cancel_button, FALSE);
-
-	/* NULLify the cancellable */
-	for (i = 0; i < NUM_TREE_VIEWS; i++) {
-		if (self->cancellable[i] == old_cancellable)
-			self->cancellable[i] = NULL;
-	}
-}
-
-static void
-set_current_operation (TotemYouTubePlugin *self, guint tree_view, GCancellable *cancellable)
-{
-	/* Cancel previous searches on this tree view */
-	if (self->cancellable[tree_view] != NULL)
-		g_cancellable_cancel (self->cancellable[tree_view]);
-
-	/* Make this the current cancellable action for the given tab */
-	g_object_weak_ref (G_OBJECT (cancellable), (GWeakNotify) cancellable_notify_cb, self);
-	self->cancellable[tree_view] = cancellable;
-
-	/* Enable the "Cancel" button if it applies to the current tree view */
-	if (self->current_tree_view == tree_view)
-		gtk_widget_set_sensitive (self->cancel_button, TRUE);
-}
-
 static void
 execute_query (TotemYouTubePlugin *self, guint tree_view, gboolean clear_tree_view)
 {
 	QueryData *data;
 
-	/* Set up the query */
-	data = g_slice_new (QueryData);
-	data->plugin = g_object_ref (self);
-	data->tree_view = tree_view;
-	data->query_cancellable = g_cancellable_new ();
-	data->t_param_cancellable = NULL;
-	data->thumbnail_cancellable = NULL;
-
-	/* Make this the current cancellable action for the given tab */
-	set_current_operation (self, tree_view, data->query_cancellable);
+	/* Cancel previous searches on this tree view */
+	if (self->cancellable[tree_view] != NULL) {
+		g_cancellable_cancel (self->cancellable[tree_view]);
+		g_object_unref (self->cancellable[tree_view]);
+	}
 
 	/* Clear the tree views */
 	if (clear_tree_view == TRUE)
 		gtk_list_store_clear (self->list_store[tree_view]);
 
+	/* Do the query */
+	self->cancellable[tree_view] = g_cancellable_new ();
+
+	data = g_slice_new (QueryData);
+	data->plugin = g_object_ref (self);
+	data->tree_view = tree_view;
+
 	if (tree_view == SEARCH_TREE_VIEW) {
-		gdata_youtube_service_query_videos_async (self->service, self->query[tree_view], data->query_cancellable,
+		gdata_youtube_service_query_videos_async (self->service, self->query[tree_view], self->cancellable[tree_view],
 							  (GDataQueryProgressCallback) query_progress_cb, data,
 							  (GAsyncReadyCallback) query_finished_cb, data);
 	} else {
-		gdata_youtube_service_query_related_async (self->service, self->playing_video, self->query[tree_view], data->query_cancellable,
+		gdata_youtube_service_query_related_async (self->service, self->playing_video, self->query[tree_view], self->cancellable[tree_view],
 							   (GDataQueryProgressCallback) query_progress_cb, data,
 							   (GAsyncReadyCallback) query_finished_cb, data);
 	}
+
+	/* Enable the "Cancel" button if it applies to the current tree view */
+	if (self->current_tree_view == tree_view)
+		gtk_widget_set_sensitive (self->cancel_button, TRUE);
 }
 
 void
@@ -984,7 +849,7 @@ search_button_clicked_cb (GtkButton *button, TotemYouTubePlugin *self)
 		 * cycles repeatedly creating new regexes for each video whose t param we resolve. */
 		/* We're looking for a line of the form:
 		 * var swfHTML = (isIE) ? "<object...econds=194&t=vjVQa1PpcFP36LLlIaDqZIG1w6e30b-7WVBgsQLLA3s%3D&rv.6.id=OzLjC6Pm... */
-		self->regex = g_regex_new ("swfHTML = .*&fmt_url_map=([^&]+)&", G_REGEX_OPTIMIZE, 0, NULL);
+		self->regex = g_regex_new ("swfHTML = .*&t=([^&]+)&", G_REGEX_OPTIMIZE, 0, NULL);
 		g_assert (self->regex != NULL);
 
 		/* Set up the GData service (needed for the tree views' queries) */
@@ -999,9 +864,6 @@ search_button_clicked_cb (GtkButton *button, TotemYouTubePlugin *self)
 		/* Set up the queries */
 		self->query[SEARCH_TREE_VIEW] = gdata_query_new_with_limits (NULL, 0, MAX_RESULTS);
 		self->query[RELATED_TREE_VIEW] = gdata_query_new_with_limits (NULL, 0, MAX_RESULTS);
-
-		/* Lazily create the SoupSession used in resolve_t_param() */
-		self->session = soup_session_async_new ();
 	}
 
 	/* Do the query */
@@ -1012,10 +874,7 @@ search_button_clicked_cb (GtkButton *button, TotemYouTubePlugin *self)
 void
 cancel_button_clicked_cb (GtkButton *button, TotemYouTubePlugin *self)
 {
-	/* It's possible for the operation to finish (and consequently the cancellable to disappear) while the GtkButton is deciding whether the
-	 * user is actually pressing it (in its timeout). */
-	if (self->cancellable[self->current_tree_view] == NULL)
-		return;
+	g_assert (self->cancellable[self->current_tree_view] != NULL);
 
 	g_debug ("Cancelling search");
 	g_cancellable_cancel (self->cancellable[self->current_tree_view]);
@@ -1042,7 +901,7 @@ load_related_videos (TotemYouTubePlugin *self)
 }
 
 void
-notebook_switch_page_cb (GtkNotebook *notebook, gpointer *page, guint page_num, TotemYouTubePlugin *self)
+notebook_switch_page_cb (GtkNotebook *notebook, GtkNotebookPage *page, guint page_num, TotemYouTubePlugin *self)
 {
 	/* Change the tree view */
 	self->current_tree_view = page_num;
@@ -1147,3 +1006,4 @@ starting_video_cb (TotemVideoList *video_list, GtkTreePath *path, TotemYouTubePl
 
 	return TRUE;
 }
+
