@@ -181,7 +181,7 @@ typedef struct {
 
 GType	totem_object_get_type			(void);
 void    totem_object_plugins_init		(TotemObject *totem);
-void    totem_object_plugins_shutdown		(void);
+void    totem_object_plugins_shutdown		(TotemObject *totem);
 void	totem_file_opened			(TotemObject *totem,
 						 const char *mrl);
 void	totem_file_closed			(TotemObject *totem);
@@ -200,8 +200,8 @@ void	totem_action_fullscreen_toggle		(Totem *totem);
 void	totem_action_fullscreen			(Totem *totem, gboolean state);
 void	totem_action_next			(Totem *totem);
 void	totem_action_previous			(Totem *totem);
-void	totem_action_seek_time			(Totem *totem, gint64 sec);
-void	totem_action_seek_relative		(Totem *totem, gint64 offset);
+void	totem_action_seek_time			(Totem *totem, gint64 msec, gboolean accurate);
+void	totem_action_seek_relative		(Totem *totem, gint64 offset, gboolean accurate);
 double	totem_get_volume			(Totem *totem);
 void	totem_action_volume			(Totem *totem, double volume);
 void	totem_action_volume_relative		(Totem *totem, double off_pct);
